#ifndef _LED_INTERFACE_
#define _LED_INTERFACE_
//-------------------------------------------------------------------------------------------------
#include "mraa.hpp"
#include "ppp_link.hpp"
#include <exception>
//-------------------------------------------------------------------------------------------------
namespace led_interface {
	
	class led_interface {
		
		public:
			led_interface();
			~led_interface();
			


			/*
			 * The function checks the connection between the CCU and the MDU.
			 * If the connection is lost, a connection_error_exception is thrown.
			 */
			void check_connection();



			/*
			 * The function sends a "set luminance" message to the MDU and waits for the acknowledge.
			 * If no response comes back within 10 seconds, the underlying PPP link throws a uart_timeout_exception.
			 * If the received acknowledge is not a valid acknowledge packet, a command_not_acknowledged_exception is thrown.
			 * In case of a checksum error in the received packet, a checksum_error_exception is thrown.
			 * If the acknowledge received signals a transmission error (checksum error response),
			 * a checksum_error_response_exception is thrown.
			 *
			 * If the passed luminance value is greater than 254, an invalid_luminance_exception is thrown.
			 */
			void set_luminance(unsigned char luminance);


			/*
			 * ...
			 */
			unsigned char get_luminance();



			/*
			 * The function sends a "set fade" message to the MDU and waits for the acknowledge.
			 * If no response comes back within 10 seconds, the underlying PPP link throws a uart_timeout_exception.
			 * If the received acknowledge is not a valid acknowledge packet, a command_not_acknowledged_exception is thrown.
			 * In case of a checksum error in the received packet, a checksum_error_exception is thrown.
			 * If the acknowledge received signals a transmission error (checksum error response),
			 * a checksum_error_response_exception is thrown.
			 */
			void set_fade(unsigned char fade);



			/*
			 * The function sends a "get U" request to the MDU and waits for the response.
			 * If no response comes back within 10 seconds, the underlying PPP link throws a uart_timeout_exception.
			 * If the received response is not a valid response packet, an invalid_response_exception is thrown.
			 * In case of a checksum error in the received packet, a checksum_error_exception is thrown.
			 * If the response received signals a transmission error (checksum error response),
			 * a checksum_error_response_exception is thrown.
			 *
			 * uid stores the ID of the LED string, whose voltage is intended to be read (1-3).
			 * In case of invalid parameter passing, an invalid_channel_id_exception is thrown.
			 * In case of a successful response, the function returns the LED string voltage (V).
			 */
			float get_u(unsigned char uid);



			/*
			 * The function sends a "get I" request to the MDU and waits for the response.
			 * If no response comes back within 10 seconds, the underlying PPP link throws a uart_timeout_exception.
			 * If the received response is not a valid response packet, an invalid_response_exception is thrown.
			 * In case of a checksum error in the received packet, a checksum_error_exception is thrown.
			 * If the response received signals a transmission error (checksum error response),
			 * a checksum_error_response_exception is thrown.
			 *
			 * iid stores the ID of the LED string, whose current is intended to be read (1-3).
			 * In case of invalid parameter passing, an invalid_channel_id_exception is thrown.
			 * In case of a successful response, the function returns the LED string current (mA).
			 */
			float get_i(unsigned char iid);
		
		private:
			ppp_link::ppp_link* ppp;
			unsigned char output_packet_buffer[32];
			unsigned char output_packet_size;
			unsigned char input_packet_buffer[32];
			unsigned char input_packet_size;
			void set_checksum();
			bool check_checksum();
			void check_command_acknowledge();
			void check_for_checksum_error_response();
		
	};



	struct connection_error_exception : public std::exception {
		const char* what() const throw() { return "Connection error!"; }
	};

	struct command_not_acknowledged_exception : public std::exception {
		const char* what() const throw() { return "Command not acknowledged!"; }
	};
	
	struct invalid_luminance_exception : public std::exception {
		const char* what() const throw() { return "Invalid luminance!"; }
	};
	
	struct checksum_error_exception : public std::exception {
		const char* what() const throw() { return "Checksum error!"; }
	};
	
	struct checksum_error_response_exception : public std::exception {
		const char* what() const throw() { return "Checksum error response!"; }
	};
	
	struct invalid_response_exception : public std::exception {
		const char* what() const throw() { return "Invalid response!"; }
	};
	
	struct invalid_channel_id_exception : public std::exception {
		const char* what() const throw() { return "Invalid channel ID!"; }
	};
	
}
//-------------------------------------------------------------------------------------------------
#endif // _LED_INTERFACE_
//-------------------------------------------------------------------------------------------------
