#ifndef _YITRAN_HOST_
#define _YITRAN_HOST_
//-------------------------------------------------------------------------------------------------
#include "mraa.hpp"
#include "yitran_packet.hpp"
#include "yitran_node_descriptor.hpp"
#include <exception>
//-------------------------------------------------------------------------------------------------
namespace yitran_host {

	typedef enum runtime_indication {
		connectivity_status_with_rs,
		new_connection_to_nc,
		disconnected_from_nc,
		connected_to_nc,
		rx_intranetworking_packet,
		unknown_indication
	} runtime_indication;



	typedef enum disconnect_reason {
		parent_unstable,
		nvr_nack,
		infinity,
		init,
		cant_start_timer,
		nvr_refused,
		nvr_enq,
		invalid_node_id,
		disconnected_by_application_request,
		unknown_reason
	} disconnect_reason;



	class yitran_host {

		public:
			yitran_host(bool use_console_port);
			virtual ~yitran_host();
			void reset();



			/*
			 * The function sends a single YITRAN packet. The checksum is calculated automatically.
			 */
			void send_packet(const yitran_packet& packet_2_send);



			/*
			 * The function receives a single packet and it checks the checksum.
			 * The bytes received before the command start symbol are discarded.
			 * The decoded packet is stored into recently_received_packet.
			 */
			void get_packet(unsigned timeout_ms);



			/*
			 * The function waits for the following indications:
			 * 	- connectivity_status_with_rs: The NC may receive an indication that a connection with an RS became invalid/valid.
			 * 	- new_connection_to_nc: The NC may receive an indication that a new RS connected to it.
			 * 	- disconnected_from_nc: An RS may receive an indication that it is disconnected from the NC.
			 * 	- connected_to_nc: An RS may receive an indication that it is connected to the NC.
			 * 	- rx_intranetworking_packet: NC and RS may receive an indication that a new RX intranetworking packet has been received.
			 * 	The function returns the type of the indication received. If any other type of indication is received, the function returns unknown_indication.
			 *
			 * 	In case of timeout, the function throws a packet_reception_timeout_exception.
			 *
			 * 	The received indication packet is stored in recently_received_packet.
			 *
			 */
			runtime_indication wait_for_runtime_indication(unsigned timeout_ms);



			/*
			 * After receiving a "RX Intranetworking Packet" indication, this function may be used to
			 * get the relevant information stored in the payload (command data) field of the indication.
			 * Relevant information:
			 * 		- origin ID -> recently_received_packet.origin_id
			 * 		- final ID -> recently_received_packet.final_id
			 * 		- user payload -> recently_received_packet.user_payload
			 */
			void decode_rx_intranetworking_packet_indication();



			/*
			 * The function receives a single packet. If the received packet is a reset response,
			 * the function returns true, otherwise it returns false.
			 */
			bool wait_for_reset_response(unsigned timeout_ms);



			/*
			 * The function receives a single packet. If the received packet is a "Go Online" response,
			 * the function returns true, otherwise it returns false.
			 */
			bool wait_for_go_online_response(unsigned timeout_ms);



			/*
			 * The function sends a NOP request and waits for the response.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 */
			void acknowledged_nop(unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the network size to size.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * Return value:
			 *   true, if the status field of the response indicates that the command was executed successfully
			 *   false, if the status field of the response indicates that the command execution failed
			 */
			bool acknowledged_set_network_size(unsigned char size, unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the network size to size.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the "status" field.
			 * If the "status" field indicates command error, the function returns false immediately.
			 * If the "status" field indicates a successful command, the function sends a "Get Device Parameters" request,
			 * reading the network size.
			 * Thereafter, the function waits for the response.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the network size value.
			 * If the network size is set to size, the function returns true, otherwise it returns false.
			 */
			bool verified_set_network_size(unsigned char size, unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the node ID to node_id.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * Return value:
			 *   true, if the status field of the response indicates that the command was executed successfully
			 *   false, if the status field of the response indicates that the command execution failed
			 */
			bool acknowledged_set_node_id(short node_id, unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the node ID to node_id.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the "status" field.
			 * If the "status" field indicates command error, the function returns false immediately.
			 * If the "status" field indicates a successful command, the function sends a "Get Device Parameters" request,
			 * reading the node ID.
			 * Thereafter, the function waits for the response.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the node ID.
			 * If the network size is set to size, the function returns true, otherwise it returns false.
			 */
			bool verified_set_node_id(short node_id, unsigned timeout_ms);



			/*
			 * The function sends a "Get Device Parameters" request addressing operation mode.
			 * Thereafter, the function waits for the response for timeout_ms.
			 * In case of timeout, a packet_reception_timeout_exception is thrown.
			 * If the response is successfully received, the response is stored into recently_received_packet.
			 * return value:
			 *   true, if the operation mode is set to NC
			 *   false, if the operation mode is set to RS
			 */
			bool get_operation_mode(unsigned timeout_ms);



			/*
			 * The function sends a "Get Device Parameters" request addressing network size.
			 * Thereafter, the function waits for the response for timeout_ms.
			 * In case of timeout, a packet_reception_timeout_exception is thrown.
			 * If the response is successfully received, the response is stored into recently_received_packet.
			 * return value: Network size
			 */
			unsigned char get_network_size(unsigned timeout_ms);



			/*
			 * The function sends a "Get Device Parameters" request addressing node ID.
			 * Thereafter, the function waits for the response for timeout_ms.
			 * In case of timeout, a packet_reception_timeout_exception is thrown.
			 * If the response is successfully received, the response is stored into recently_received_packet.
			 * return value: node ID
			 */
			short get_node_id(unsigned timeout_ms);



			/*
			 * The function sends a "Save Device Parameters" request and it waits for response for timeout_ms.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * Return value:
			 *   true, if the status field of the response indicates that the command was executed successfully
			 *   false, if the status field of the response indicates that the command execution failed
			 */
			bool acknowledged_save_device_parameters(unsigned timeout_ms);



			/*
			 * The function sends a "Go Online" request and it waits for response for timeout_ms.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * Return value:
			 *   true, if the status field of the response indicates that the command was executed successfully
			 *   false, if the status field of the response indicates that the command execution failed
			 */
			bool acknowledged_go_online(unsigned timeout_ms);



			/*
			 * The function sends a "Packet TX" command.
			 * If the user payload size is greater than 245, the function throws a invalid_user_payload_size_exception.
			 *
			 * "Packet TX" command parameters:
			 * Data Service Type: Intranetworking Unicast
			 * Priority: Normal
			 * Ack Service: Ack required
			 * Hops: 8
			 * Gain: 7
			 * Tag: 0
			 * Encrypt: 0
			 * Destination port: 0
			 * target ID LSB: destination_id LSB
			 * target ID MSB: destination_id MSB
			 * user payload: user_payload
			 *
			 * The function waits for "Packet TX Admission" and "Packet TX Transmission" responses. Any other types of packets are discarded while waiting for these.
			 * In case of timeout, the function throws a packet_reception_timeout_exception.
			 * If any of the responses indicates an error (the packet tx request is rejected or the transmission fails), the function returns false.
			 * If the transmission is successful, the function return true.
			 */
			bool acknowledged_packet_tx(short destination_id, unsigned char* user_payload, unsigned char user_payload_size, unsigned timeout_ms);



			/*
			 * The function sends a "Packet TX" command.
			 * If the user payload size is greater than 245, the function throws a invalid_user_payload_size_exception.
			 *
			 * "Packet TX" command parameters:
			 * Data Service Type: Intranetworking Unicast
			 * Priority: Normal
			 * Ack Service: Ack required
			 * Hops: 8
			 * Gain: 7
			 * Tag: 0
			 * Encrypt: 0
			 * Destination port: 0
			 * target ID LSB: destination_id LSB
			 * target ID MSB: destination_id MSB
			 * user payload: user_payload
			 *
			 */
			void non_acknowledged_packet_tx(short destination_id, unsigned char* user_payload, unsigned char user_payload_size);



			/*
			 * The function prints the recently received packet to the standard output.
			 */
			void print_recently_received_packet();



			yitran_packet recently_received_packet;


		protected:
			bool get_byte(unsigned timeout_ms, char* buffer);



			/*
			 * The functions sends a NOP request.
			 */
			void send_nop();



			/*
			 * The function sends a "Set Operation Mode" request setting the operation mode to NC.
			 */
			void set_nc_operation_mode();



			/*
			 * The function sends a "Set Operation Mode" request setting the operation mode to RS.
			 */
			void set_rs_operation_mode();



			/*
			 * The functions sets the NC database size.
			 */
			void set_nc_database_size(unsigned char size);



			/*
			 * The functions sets the network size.
			 */
			void set_network_size(unsigned char size);



			/*
			 * The function sets the node ID.
			 */
			void set_node_id(short node_id);



			/*
			 * The function sends a "Save Device Parameters" request.
			 */
			void save_device_parameters();



			/*
			 * The function sends a "Go Online" request.
			 */
			void go_online();



			mraa::Uart* yitran_uart;
			mraa::Gpio* yitran_reset_pin;

	};


	struct checksum_error_exception : public std::exception {
		const char* what() const throw() { return "Checksum error in received packet. Packet discarded"; }
	};

	struct packet_reception_timeout_exception : public std::exception {
		const char* what() const throw() { return "Packet reception timed out"; }
	};

	struct reset_response_timeout_exception : public std::exception {
		const char* what() const throw() { return "Reset response timed out"; }
	};

	struct invalid_nc_database_index_exception : public std::exception {
		const char* what() const throw() { return "Invalid NC database index"; }
	};

	struct invalid_user_payload_size_exception : public std::exception {
		const char* what() const throw() { return "Invalid user payload size"; }
	};

	struct packet_tx_admission_failed_exception : public std::exception {
		const char* what() const throw() { return "Packet TX Admission failed"; }
	};

}
//-------------------------------------------------------------------------------------------------
#endif // _YITRAN_HOST_
//-------------------------------------------------------------------------------------------------
