#ifndef _YITRAN_NC_
#define _YITRAN_NC_
//-------------------------------------------------------------------------------------------------
#include "mraa.hpp"
#include "yitran_packet.hpp"
#include "yitran_node_descriptor.hpp"
#include "yitran_host.hpp"
#include <exception>
//-------------------------------------------------------------------------------------------------
namespace yitran_host {

	typedef struct rs_hk_data_set {
		float u1;
		float u2;
		float u3;
		float u4;
		float i1;
		float i2;
		float i3;
		float i4;
		float temperature;
		float humidity;
		unsigned char luminance;
		short pylon_id;
		char maintenance_required_flag;
	} rs_hk_data_set;

	class yitran_nc : public yitran_host {

		public:
			yitran_nc(bool use_console_port);
			~yitran_nc();

			/*
			 * The function sends a 'data exchange request' to the RS node with logical id of 'destination_id'.
			 * The 'data exchange request' indicates that the NC node wants to access some data provided by the RS node or to adjust the luminance of it.
			 * The 'set luminance commad' consists of the following bytes:
			 * byte #0 (sent first): 0x80: command header
			 * byte #1: flag indicating whether the luminance shall be adjusted or not
			 * byte #2: new luminance value
			 *
			 * This function sends the data exchange request with byte #1 set to 0x01.
			 */
			void send_set_luminance_command(const short destination_id, const unsigned char new_luminance);



			/*
			 * The function sends a 'data exchange request' to the RS node with logical id of 'destination_id'.
			 * The 'data exchange request' indicates that the NC node wants to access some data provided by the RS node or to adjust the luminance of it.
			 * The 'set luminance commad' consists of the following bytes:
			 * byte #0 (sent first): 0x80: command header
			 * byte #1: flag indicating whether the luminance shall be adjusted or not
			 * byte #2: new luminance value
			 *
			 * * This function sends the data exchange request with byte #1 set to 0x00.
			 */
			void send_full_hk_request_command(const short destination_id);



			/*
			 * When an RS node receives a 'data exchange request', it'll send a set of HK data back to the NC node, and it'll adjust the luminance, if it's needed (see byte #1 in 'data exchange request').
			 * When the NC node receives the reply packet to a 'data exchange request', it shall decode it by calling this function.
			 * This function processes the 'recently_received_packet' of the yitran_nc object (publicly inherited from yitran_host).
			 * The function returns a complete 'rs_hk_data_set' struct with the recently received HK data.
			 */
			rs_hk_data_set decode_data_exchange_reply();



			/*
			 * The function initializes the Yitran modem as an NC node, saves the device parameters into the non-volatile memory, resets the modem and sends a go-online request.
			 */
			bool reset_and_go_online(unsigned network_size);


			/*
			 * The function initializes the NC node and the PLC network automatically. If initial_number_of_rss nodes are connected,
			 * the function returns true. In this case, rs_address_table is ready to use.
			 * The network_size stores the number of nodes in the PLC network.
			 * In case of a timeout, the function throws a packet_reception_timeout_exception.
			 */
			bool initialize_and_establish_a_network(unsigned char initial_number_of_rss, unsigned network_size);



			/*
			 * After the PLC network is established, this function may be used to send an indication to the RSs
			 * that they can start to send and receive user packets.
			 */
			void send_data_traffic_enabled_indication();



			/*
			 * After receiving a "Connectivity Status with RS" indication, this function may be used to
			 * get the relevant information stored in the payload (command data) field of the indication.
			 * Relevant information:
			 * 		- node ID: The ID of the node sending the indication. -> nd.node_logical_id
			 * 		- Connectivity status: The new connectivity status of the node. -> nd.connectivity_status
			 */
			void decode_connectivity_status_with_rs_indication(yitran_node_descriptor& nd);



			/*
			 * After receiving a "New Connection to NC" indication, this function may be used to
			 * get the relevant information stored in the payload (command data) field of the indication.
			 * Relevant information:
			 * 		- node ID: The logical ID of the node sending the indication. -> nd.node_logical_id
			 * 		- parent ID: The logical ID of the node's parent. -> nd.parent_id
			 */
			void decode_new_connection_to_nc_indication(yitran_node_descriptor& nd);



			/*
			 * The function receives a single packet. If the received packet is a "Network ID Assigned" indication,
			 * the function returns true, otherwise it returns false.
			 */
			bool wait_for_network_id_assigned_indication(unsigned timeout_ms);



			/*
			 * The function receives a single packet. If the received packet is a "New Connection to NC" indication,
			 * the function returns true, otherwise it returns false.
			 * If the received packet is a "New Connection to NC" indication, the parameters of the new node are stored into nd as follows:
			 * 		- node ID: received
			 * 		- parent ID: received
			 * 		- serial number: 0
			 * 		- connectivity status: 1
			 */
			bool wait_for_new_connection_to_nc_indication(yitran_node_descriptor& nd, unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the operation mode to NC.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * Return value:
			 *   true, if the status field of the response indicates that the command was executed successfully
			 *   false, if the status field of the response indicates that the command execution failed
			 */
			bool acknowledged_set_nc_operation_mode(unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the operation mode to NC.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the "status" field.
			 * If the "status" field indicates command error, the function returns false immediately.
			 * If the "status" field indicates a successful command, the function sends a "Get Device Parameters" request,
			 * reading the operation mode.
			 * Thereafter, the function waits for the response.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the operation mode value.
			 * If the operation mode is set to NC, the function returns true, otherwise it returns false.
			 */
			bool verified_set_nc_operation_mode(unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the NC database size to size.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * Return value:
			 *   true, if the status field of the response indicates that the command was executed successfully
			 *   false, if the status field of the response indicates that the command execution failed
			 */
			bool acknowledged_set_nc_database_size(unsigned char size, unsigned timeout_ms);



			/*
			 * The function sends a "Set Device Parameters" request setting the NC database size to size.
			 * Thereafter, the function waits for the response and checks the "status" field.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the "status" field.
			 * If the "status" field indicates command error, the function returns false immediately.
			 * If the "status" field indicates a successful command, the function sends a "Get Device Parameters" request,
			 * reading the NC database size.
			 * Thereafter, the function waits for the response.
			 * If no response comes back, the function throws a packet_reception_timeout_exception.
			 * If a response is received, the function checks the NC database size value.
			 * If the NC database size is set to size, the function returns true, otherwise it returns false.
			 */
			bool verified_set_nc_database_size(unsigned char size, unsigned timeout_ms);



			/*
			 * The function sends a "Get Device Parameters" request addressing NC database size.
			 * Thereafter, the function waits for the response for timeout_ms.
			 * In case of timeout, a packet_reception_timeout_exception is thrown.
			 * If the response is successfully received, the response is stored into recently_received_packet.
			 * return value: NC database size
			 */
			unsigned char get_nc_database_size(unsigned timeout_ms);



			/*
			 * The function sends a "Get NC Database Size" request.
			 * Thereafter, the function waits for the response for timeout_ms.
			 * In case of timeout, a packet_reception_timeout_exception is thrown.
			 * If the response is successfully received, the response is stored into recently_received_packet.
			 * return value: current size of the NC database
			 */
			unsigned char get_current_nc_database_size(unsigned timeout_ms);



			/*
			 * The function sends a "Get Node Information" request. The requested node is identified based on the NC database index.
			 * If the NC database index is invalid (greater than "NC database size"), the function throws an invalid_nc_database_index_exception.
			 * If the response's status field indicates a successful command, the function returns true, otherwise it returns false.
			 * In case of timeout, a packet_reception_timeout_exception is thrown.
			 * The node information is stored into nd.
			 */
			bool get_node_information(unsigned char node_index, yitran_node_descriptor& nd, unsigned timeout_ms);



			short rs_address_table[256];
			unsigned number_of_connected_rss;



		private:
			//...

	};


	struct yitran_nc_not_a_valid_data_exchange_reply_exception : public std::exception {
		const char* what() const throw() { return "It's not a valid data exchange reply!"; }
	};
}
//-------------------------------------------------------------------------------------------------
#endif // _YITRAN_HOST_
//-------------------------------------------------------------------------------------------------
